/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.exml.editor;

/**
 * Used to create a new editor for a given extension.
 * It also has a description
 */
public interface EditorTemplate {
  /**
   * The new editor template type.
   */
  public final static int EDITOR_TEMPLATE = 1;
  
  /**
   * The classic type of template represented by a file on HDD.
   */
  public final static int FILE_TEMPLATE = 2;
  
  /**
   * The archive template type.
   */
  public final static int ARCHIVE_TEMPLATE = 3;
  
  /**
   *  Return the template description.
   *
   * @return The template description.
   */
  String getDescription();
  
  /**
   * Return the template extension.
   *
   * @return The template extension.
   */
  String getExtension();
  
  /**
   * @return A description from where this template was loaded.
   */
  String getSource();
  
  /**
   * @return The template type. Currently one of: EDITOR_TEMPLATE, FILE_TEMPLATE or ARCHIVE_TEMPLATE.
   */
  int getTemplateType();
  
  /**
   * Clone this editor template.
   * 
   * @return The clone or null if unsuccessful.
   */
  Object clone();
  
  /**
   * Return the template name.
   * 
   * @return The template name.
   */
  String getName();

  /**
   * Return additional information about this template (e.g. Framework, Path etc).
   * 
   * @return Additional information.
   */
  String getAdditionalInformation();
  
  /**
   * @return <code>true</code> if the template can be customized.
   */
  boolean isCustomizable();
  
  /**
   * Get the ID representing the page used for customizing the template.
   * 
   * @return The ID representing the page used for customizing the template.
   */
  String getCustomizePageID();

  /**
   * @return The caret position to be set after loading the template.
   */
  int getCaretPosition();
}