/*
 *  The Syncro Soft SRL License
 *
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  3. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  4. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  5. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.exml.plugin.document;

import java.awt.Frame;

import javax.swing.text.Document;

import ro.sync.exml.workspace.api.standalone.StandalonePluginWorkspace;


/**
 * Support implementation of the PluginContext interface.
 * 
 * @author iulian_velea
 * @created Jun 21, 2004
 * 
 */
public class DocumentPluginContextImpl implements DocumentPluginContext {
  /**
   * The current document.
   */
  protected Document document;
  
  /**
   * The parent frame.
   */
  protected Frame frame;

  /**
   * Access to Oxygen's workspace.
   */
  private StandalonePluginWorkspace wsAccess;

  /**
   * Constructor for an DocumentPluginContext with no document. 
   */
  public DocumentPluginContextImpl() {
    //DO nothing.
  }
  
  /**
   * Creates the plugin context.
   * 
   * @param document The current document.
   * @param frame The parent frame for the swing components.
   * @param wsAccess Access to Oxygen's workspace
   */
  public DocumentPluginContextImpl(Document document, Frame frame, StandalonePluginWorkspace wsAccess) {
    this.document = document;
    this.frame = frame;
    this.wsAccess = wsAccess;
  }

  /**
   * Set the current document.
   * 
   * @param document The document to be set.
   */
  public void setDocument(Document document) {
    this.document = document;
  }

  /**
   *  Sets the parent frame for the PluginContextImpl.
   *
   *@param  frame  The new frame value.   
   */
  public void setFrame(Frame frame) {
    this.frame = frame;
  }
  
  /**
   * Get the current document.
   * 
   * @return the current document.
   */
  public Document getDocument() {
    return document;
  }

  /**
   * Get the frame for the DocumentPluginContextImpl.
   * 
   * @return the current frame.
   */
  public Frame getFrame() {
    return frame;
  }

  /**
   * @see ro.sync.exml.plugin.document.DocumentPluginContext#getPluginWorkspace()
   */
  public StandalonePluginWorkspace getPluginWorkspace() {
    return wsAccess;
  }
}
