/*
 *  The Syncro Soft SRL License
 *
 *  Copyright (c) 1998-2007 Syncro Soft SRL, Romania.  All rights
 *  reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistribution of source or in binary form is allowed only with
 *  the prior written permission of Syncro Soft SRL.
 *
 *  2. Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 *  3. Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the
 *  distribution.
 *
 *  4. The end-user documentation included with the redistribution,
 *  if any, must include the following acknowledgment:
 *  "This product includes software developed by the
 *  Syncro Soft SRL (http://www.sync.ro/)."
 *  Alternately, this acknowledgment may appear in the software itself,
 *  if and wherever such third-party acknowledgments normally appear.
 *
 *  5. The names "Oxygen" and "Syncro Soft SRL" must
 *  not be used to endorse or promote products derived from this
 *  software without prior written permission. For written
 *  permission, please contact support@oxygenxml.com.
 *
 *  6. Products derived from this software may not be called "Oxygen",
 *  nor may "Oxygen" appear in their name, without prior written
 *  permission of the Syncro Soft SRL.
 *
 *  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *  DISCLAIMED.  IN NO EVENT SHALL THE SYNCRO SOFT SRL OR
 *  ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */
package ro.sync.sample.plugin.customprotocol;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FilterOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLStreamHandler;

/**
 * Handler for the file2 protocol
 */
public class CustomProtocolHandler extends URLStreamHandler {
  /**
   * Connection class for file2
   */
  private static class File2Connection extends URLConnection {
    /**
     * Construct the connection
     * 
     * @param url
     *          The URL
     */
    protected File2Connection(URL url) {

      super(url);
      // Allow output
      setDoOutput(true);
    }
    
    /**
     * @see java.net.URLConnection#getURL()
     */
    @Override
    public URL getURL() {
      URL toReturn = super.getURL();
      String content = toReturn.toString();
      if(content.indexOf("?") != -1) {
        //Remove the parameters part
        if(content.indexOf("#") != -1) {
          //But keep the anchor in place
          content = content.substring(0, content.indexOf("?")) + content.substring(content.indexOf("#"), content.length());
        } else {
          content = content.substring(0, content.indexOf("?"));
        }
      }
      return toReturn;
    }

    /**
     * Returns an input stream that reads from this open connection.
     * 
     * @return the input stream
     */
    @Override
    public InputStream getInputStream() throws IOException {
      File file = getCanonicalFileFromFileUrl(url);
      final FileInputStream fis = new FileInputStream(file);
//      InputStream is = new InputStream() {
//        @Override
//        public int read() throws IOException {
//          try {
//            Thread.sleep(200);
//          } catch (InterruptedException e) {
//            e.printStackTrace();
//          }
//          return fis.read();
//        }
//      };
      return fis;
    }

    /**
     * Returns an output stream that writes to this connection.
     * 
     * @return the output stream
     */
    @Override
    public OutputStream getOutputStream() throws IOException {
      File file = getCanonicalFileFromFileUrl(url);
      final FileOutputStream fos = new FileOutputStream(file);
      OutputStream os = new FilterOutputStream(fos) {
        /**
         * @see java.io.FilterOutputStream#write(byte[])
         */
        @Override
        public void write(byte[] b) throws IOException {
//          System.err.println("WRITE " + b.length);
//          try {
//            Thread.sleep(5);
//          } catch (InterruptedException e) {
//            e.printStackTrace();
//          }
          super.write(b);
//          System.err.println("WRITE OVER " + b.length);
        }
        /**
         * @see java.io.FilterOutputStream#write(byte[], int, int)
         */
        @Override
        public void write(byte[] b, int off, int len) throws IOException {
//          System.err.println("WRITE " + len);
//          try {
////            Thread.sleep(5);
//          } catch (InterruptedException e) {
//            e.printStackTrace();
//          }
          super.write(b, off, len);
//          System.err.println("WRITE OVER " + len);
        }
        /**
         * @see java.io.FilterOutputStream#write(int)
         */
        @Override
        public void write(int b) throws IOException {
//          System.err.println("WRITE BYTE " + b);
          try {
            Thread.sleep(5);
          } catch (InterruptedException e) {
            e.printStackTrace();
          }

          super.write(b);
        }
      };
      return os;
    }

    /**
     * Opens a communications link to the resource referenced by this URL, if such a connection has
     * not already been established.
     */
    @Override
    public void connect() throws IOException {
      this.connected = true;
    }
    
    /**
     * @see java.net.URLConnection#getContentLength()
     */
    @Override
    public int getContentLength() {
      File file = getCanonicalFileFromFileUrl(url);
      return (int) file.length();
    }
  }

  /**
   * Creates and opens the connection
   * 
   * @param u
   *          The URL
   * @return The connection
   */
  @Override
  protected URLConnection openConnection(URL u) throws IOException {
    URLConnection connection = new File2Connection(u);
    return connection;
  }
  
  /**
   * On Windows names of files from network neighborhood must be corrected before open.
   * 
   * @param url
   *          The file URL.
   * @return The cannonical or absolute file.
   * @throws IllegalArgumentException
   *           if the URL is not file.
   */
  public static File getCanonicalFileFromFileUrl(URL url) {
    File file = null;
    if (url == null) {
      throw new NullPointerException("The URL cannot be null.");
    }
    try {
      URI uri = new URI(url.toString().replace("cproto", "file"));
      file = new File(uri);
    } catch (URISyntaxException e) {
      e.printStackTrace();
    }
    return file;
  }
}
